package com.jiashi;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.Objects;

/**
 * @author 文件路径工具
 */
public class FilePathUtil {
    private static final Logger logger = LoggerFactory.getLogger(FilePathUtil.class);

    /**
     * 当前所在文件路径
     *
     * @return 当前所在文件路径
     */

    public static String currentPath() {
        try {
            File file = new File(".");
            return file.getCanonicalPath();
        } catch (Exception ex) {
            return null;
        }
    }

    /**
     * 临时缓存文件夹
     *
     * @return 临时缓存文件夹
     */
    public static String tempPath() {
        String tempPath;
        try {
            tempPath = currentPath() + File.separator + "temp";
        } catch (Exception ex) {
            tempPath = "C:\\docus\\temp";
        }
        mkdirs(tempPath);
        return tempPath;
    }

    /**
     * 根据文件夹路径创建文件目录
     * @param path 文件目录路径
     */
    public static void mkdirs(String path) {
        path = Objects.requireNonNull(path);
         mkdirs(new File(path));
    }

    /**
     * 创建文件目录
     * @param file 文件目录
     */
    public static void mkdirs(File file) {
        if (!file.exists()) {
            if (!file.mkdirs()) {
                // Not processed
                logger.warn("mkdirs error ： {}", file);
            }
        }
    }


    /**
     * 递归删除目录下的所有文件及子目录下所有文件
     * @param dir 将要删除的文件目录
     */
    public static void deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            if (children!=null) {
                //递归删除目录中的子目录下
                for (String child : children) {
                    deleteDir(new File(dir, child));
                }
            }
        }
        // 目录此时为空，可以删除
         dir.delete();
    }

    /**
     * 递归删除目录下的所有文件及子目录下所有文件
     * @param dir 将要删除的文件目录路径
     */
    public static void deleteDir(String dir) {
        deleteDir(new File(dir));
    }


}
