package com.jiashi;

import com.google.gson.Gson;
import com.jiashi.service.LianZhongUploadInfo;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

public class FileUploader {


    public static void uploadFilesWithParams(List<File> files, String uploadUrl, List<FormField> params) throws IOException {
        OkHttpClient client = new OkHttpClient.Builder()
                .connectTimeout(600, TimeUnit.SECONDS)
                .readTimeout(600, TimeUnit.SECONDS)
                .writeTimeout(600, TimeUnit.SECONDS)
                .build();
        MultipartBody.Builder builder = new MultipartBody.Builder();
        for (File file : files) {
            RequestBody body = RequestBody.create(MediaType.parse("multipart/form-data"), file);
//                RequestBody body = RequestBody.create(MediaType.parse("image/jpeg"), file);
            builder.addFormDataPart("files", file.getName(), body);
        }
        for(FormField formField:params){
            builder.addFormDataPart(formField.key, formField.value);
        }
        // 下行是上传的json的对象，其中上面的fileName和下行的fromData需和后台接口名字一致
        RequestBody requestBody = builder.setType(MultipartBody.FORM).build();
        final Request request = new Request.Builder()
                .url(uploadUrl)
                .post(requestBody)
                .build();

        Response response = client.newCall(request).execute();
        String responseBody = response.body().string();

        // 输出请求结果
        System.out.println(responseBody);
    }

    public static void main(String[] args) throws IOException {
        List<File> files = new ArrayList<>();
        File file1 = new File("C:\\Users\\wyb\\Pictures\\head.jpg");
        File file2 = new File("C:\\Users\\wyb\\Pictures\\毒王90.jpg");
        files.add(file1);
        files.add(file2);

//        files.add(new File("C:\\jiahsi-saomiao\\413425_刘燊杨_20221009_004624\\16763947060420811242.jpg"));
        LianZhongUploadInfo.PatientInfo patientInfo = new LianZhongUploadInfo.PatientInfo();
        patientInfo.setInpatientNo("wyb-lianzhong001");
        patientInfo.setAdmissTimes(1);
        patientInfo.setDisDate("2025-01-01 00:00:00");
        patientInfo.setName("厦门中医院联众测试");

        LianZhongUploadInfo.FileInfo fileInfo1 = new LianZhongUploadInfo.FileInfo();
        fileInfo1.setSort(1);
        fileInfo1.setFileTitle("head");
        fileInfo1.setUploadFileName(file1.getName());
        fileInfo1.setAssortId("wyb-lianzhong");

        LianZhongUploadInfo.FileInfo fileInfo2 = new LianZhongUploadInfo.FileInfo();
        fileInfo2.setSort(2);
        fileInfo2.setFileTitle("head2");
        fileInfo2.setUploadFileName(file2.getName());
        fileInfo2.setAssortId("wyb-lianzhong-2");

        ArrayList<LianZhongUploadInfo.FileInfo> fileInfos = new ArrayList<>();
        fileInfos.add(fileInfo1);
        fileInfos.add(fileInfo2);


        LianZhongUploadInfo uploadInfo = new LianZhongUploadInfo();
        uploadInfo.setPatientInfo(patientInfo);
        uploadInfo.setFileInfos(fileInfos);

        // 额外的表单字段参数
        List<FormField> params = new ArrayList<>();
        params.add(new FormField("uploadFileParams", new Gson().toJson(uploadInfo)));

        // 上传URL
        String uploadUrl = "http://127.0.0.1:9511/lianzhong/batchFileUploadJpg";

        // 执行批量上传
        uploadFilesWithParams(files, uploadUrl, params);
    }

    // 用于表单字段的辅助类
    static class FormField {
        String key;
        String value;

        FormField(String key, String value) {
            this.key = key;
            this.value = value;
        }
    }
}
